import requests
from threading import Thread
import time
import json
from .stack import Stack, Blueprint
from .cluster import Cluster

class Client(object):
    def __init__(self,
        url='http://localhost:8080',
        username='admin',
        passwd='admin',
        retry_refused=False,
        retry_interval=3,
        retry_timeout=90,
        footprint=True,
    ):
        self.url=url+'/api/v1'
        self.username=username
        self.passwd=passwd
        self.retry_interval=retry_interval
        self.retry_timeout=retry_timeout
        self.retry_refused=retry_refused
        self.footprint=footprint
        self._stacks=None
    #curl -u admin:passwd  -H 'X-Requested-By: ambari' -X PUT -d '{"RequestInfo": {"context" :"Stop service "}, "Body": {"ServiceInfo": {"state": "INSTALLED"}}}' http://<AMBARI_SERVER_HOSTNAME>:8080/api/v1/clusters/<CLUSTER_NAME>/services/<Service_name>
    def _request(self,url,data=None,call_method=None,status_code=None,bad_code_retry=False):
        kwargs={
            'url':url,
            'headers': {'X-Requested-By': 'ambari'},
            'auth': (self.username,self.passwd)
        }
        if data:
            kwargs['data']=json.dumps(data)
            if not call_method:
                call_method=requests.put
        elif not call_method:
            call_method=requests.get
        mustend = time.time() + self.retry_timeout
        while time.time() < mustend:
            try:
                response = call_method(**kwargs)
            except requests.ConnectionError as ce:
                if self.retry_refused:
                    time.sleep(self.retry_interval)
                else:
                    raise ce
            else:
                try:
                    ret = response.json()
                    # print(response.text)
                except Exception:
                    ret = {}
                condition=status_code if status_code else requests.codes.ok
                if response.status_code != condition:
                    if bad_code_retry:
                        time.sleep(self.retry_interval)
                        continue
                    raise Exception('{} {} {}: {}'.format(call_method.__name__,url,response.status_code,ret))
                if self.footprint: print(call_method.__name__,url,response.status_code)
                if 'Requests' in ret and 'status' in ret['Requests']:
                    while True:
                        time.sleep(self.retry_interval*2)
                        if self._request(ret['href'])['Requests']['request_status']!='IN_PROGRESS':
                            break
                return ret
        raise Exception('timeout',call_method.__name__,url)
    def get(self,url):
        return self._request(self.url+url)
    def put(self,url,data,status_code=None,bad_code_retry=False):
        return self._request(self.url+url,data=data,status_code=status_code,bad_code_retry=bad_code_retry)
    def create(self,url,data=None,status_code=201,bad_code_retry=False):
        return self._request(self.url+url,data=data,call_method=requests.post,status_code=status_code,bad_code_retry=bad_code_retry)
    def delete(self,url):
        return self._request(self.url+url,call_method=requests.delete)
    @property
    def stack_info(self):
        return self.get('/stacks')['items']
    @property
    def stacks(self):
        if self._stacks is None:
            self._stacks=[]
            for s in self.stack_info:
                name=s['Stacks']['stack_name']
                vs=self.get('/stacks/'+name)['versions']
                for v in vs:
                    version=v['Versions']['stack_version']
                    self._stacks.append(Stack(client=self,name=name,version=version))
        return self._stacks
    @property
    def stack(self):
        return self.stacks[-1]
    def get_stack(self, name, version):
        for s in self.stacks:
            if s.name==name and s.version==version:
                return s
        return None
    @property
    def host_info(self):
        return self.get('/hosts')['items']
    @property
    def blueprints(self):
        bs=[]
        for b in self.get('/blueprints')['items']:
            bs.append(Blueprint(client=self,name=b['Blueprints']['blueprint_name']))
        return bs
    @property
    def version_definitions(self):
        vs=[]
        for v in self.get('/version_definitions')['items']:
            vs.append(VersionDefinition(client=self,id=v['VersionDefinition']['id']))
        return vs
    def register_version_definition(self, url):
        for v in self.version_definitions:
            if v.version_url==url:
                return v
        ret = self.create('/version_definitions',data={"VersionDefinition": {"version_url": url}})
        id=ret["resources"][0]['VersionDefinition']['id']
        return VersionDefinition(self, id)
    @property
    def clusters(self):
        cs=[]
        for c in self.get('/clusters')['items']:
            cs.append(Cluster(client=self,name=c['Clusters']['cluster_name']))
        return cs
    @property
    def cluster(self):
        return self.clusters[-1]
    def create_cluster(self, name, hosts, VDF_url=None, size='typical_triple'):
        if VDF_url:
            version_definition = self.register_version_definition(VDF_url)
            stack=version_definition.stack
        else:
            stack=self.stack
            version_definition=stack.version_definition
        blueprint=getattr(stack, 'register_blueprint_{}'.format(size))()
        host_groups=[]
        l=len(blueprint.info['host_groups'])
        if l==1:
            hg_names=['master']
        else:
            hg_names=['master1','master2','slave']
        for ni in range(l):
            host_groups.append({'name': hg_names[ni], "hosts" :[{"fqdn" : hosts[ni]}]})
        for h in hosts[l:]:
            host_groups[-1]['hosts'].append({"fqdn" : h})
        data = {
            "blueprint" : blueprint.name,
            'host_groups': host_groups
        }
        if version_definition: data["repository_version_id"]=version_definition.id
        c=Cluster(client=self, name=name)
        self.create(c.url,data=data,status_code=202)
        return c

class VersionDefinition(object):
    def __init__(self,client,id):
        self.id=id
        self.client=client
        self.url='/version_definitions/{}'.format(self.id)
    @property
    def info(self):
        return self.client.get(self.url)
    @property
    def version_url(self):
        return self.info['VersionDefinition']['version_url']
    @property
    def stack(self):
        name=self.info['VersionDefinition']['stack_name']
        version=self.info['VersionDefinition']['stack_version']
        for s in self.client.stacks:
            if s.name==name and s.version==version:
                return s
        raise Exception('Stack not found')
    def delete(self):
        url=self.stack.url+'/repository_versions/{}'.format(self.id)
        return self.client.delete(url)


import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from ambari.client import Client

c=Client('http://172.31.10.184:8080')
c.cluster.start()
# for vd in c.version_definitions: vd.delete()
# for blp in c.blueprints: blp.delete()
# # c.stack.blueprint.delete()
# c.create_cluster('packone',['packone'],VDF_url='http://public-repo-1.hortonworks.com/HDP/centos7/2.x/updates/2.6.5.0/HDP-2.6.5.0-292.xml', size='minimal_single')
# for r in reversed(c.cluster.requests):
#     print(r)
#     for t in r.tasks:
#         print(t)
# print(c.cluster.services[-1].start())


class Stack(object):
    def __init__(self,client,name,version):
        self._info=None
        self.client=client
        self.name=name
        self.version=version
        self.id='{stack.name}-{stack.version}'.format(stack=self)
        self.url="/stacks/{}/versions/{}".format(self.name,self.version)
    @property
    def info(self):
        if not self._info:
            self._info=self.client.get(self.url)
        return self._info
    @property
    def services(self):
        srvs=[]
        for s in self.info['services']:
            srvs.append(Service(stack=self,name=s['StackServices']['service_name']))
        return srvs
    def get_service(self, name):
        for s in self.services:
            if s.name==name:
                return s
        return None
    def get_component(self, name):
        for s in self.services:
            for c in s.components:
                if c.name==name:
                    return c
        return None
    

    @property
    def version_definition(self):
        for v in self.client.version_definitions:
            if v.stack==self: return v
        return None
    topology={                                                                            
        'ZOOKEEPER':{
            'host_groups': {'slave': ['ZOOKEEPER_SERVER'], 'client': ['ZOOKEEPER_CLIENT']}
        },
        'AMBARI_INFRA_SOLR': {
            'host_groups': {'master2': ['INFRA_SOLR'], 'client': ['INFRA_SOLR_CLIENT']},
            'dependencies': ['ZOOKEEPER']
        },
        'AMBARI_METRICS': {
            'host_groups': {'master2': ['METRICS_COLLECTOR','METRICS_GRAFANA'], 'slave': ['METRICS_MONITOR']},
            'configurations':{'ams-grafana-env': {'metrics_grafana_password':'bigdata'}},
            'dependencies': ['ZOOKEEPER']
        },
        'HDFS': {
            'host_groups': {'master1': ['SECONDARY_NAMENODE'], 'master2': ['NAMENODE'], 'slave': ['DATANODE'], 'client': ['HDFS_CLIENT']},
            'dependencies': ['ZOOKEEPER']
        },
        'YARN': {
            'host_groups': {'master1': ['RESOURCEMANAGER'],  'master2': ['APP_TIMELINE_SERVER'], 'slave': ['NODEMANAGER'], 'client': ['YARN_CLIENT']},
            'dependencies': ['HDFS']
        },
        'HBASE': {
            'host_groups': {'master1': ['HBASE_MASTER'], 'slave': ['HBASE_REGIONSERVER'], 'client': ['HBASE_CLIENT']},
            'dependencies': ['HDFS', 'ZOOKEEPER']
        },
        'TEZ': {
            'host_groups': {'client': ['TEZ_CLIENT']},
            'dependencies': ['HDFS', 'MAPREDUCE2', 'YARN']
        },
        'MAPREDUCE2': {
            'host_groups': {'master2': ['HISTORYSERVER'], 'client': ['MAPREDUCE2_CLIENT']},
            'dependencies': ['HDFS', 'TEZ']
        },
        'HIVE': {
            'host_groups': {'master1': ['HIVE_SERVER'], 'master2': ['HIVE_METASTORE','MYSQL_SERVER'], 'client': ['HIVE_CLIENT']},
            'configurations':{'hive-site': {'javax.jdo.option.ConnectionPassword':'bigdata'}},
            'dependencies': ['MAPREDUCE2', 'TEZ', 'YARN', 'ZOOKEEPER', 'HDFS']
        },
        'SPARK2': {
            'host_groups': {'master1': ['SPARK2_JOBHISTORYSERVER'], 'client': ['SPARK2_CLIENT']},
            'dependencies': ['HDFS', 'MAPREDUCE2','YARN', 'HIVE']
        },
        'PIG': {
            'host_groups': {'client': ['PIG']}
        },
        'ZEPPELIN': {
            'host_groups': {'master1': ['ZEPPELIN_MASTER'],},
            'dependencies': ['SPARK2', 'YARN']
        },
        'KAFKA': {
            'host_groups': {'master2': ['KAFKA_BROKER']},
            'dependencies': ['ZOOKEEPER']
        },
        'STORM': {
            'host_groups': {'master1': ['STORM_UI_SERVER'], 'master2': ['NIMBUS','DRPC_SERVER'], 'slave': ['SUPERVISOR']},
            'dependencies': ['ZOOKEEPER']
        },
        'LOGSEARCH': {
            'host_groups': {'master1': ['LOGSEARCH_SERVER'], 'slave': ['LOGSEARCH_LOGFEEDER']},
            'dependencies': ['AMBARI_INFRA_SOLR']
        },
        'OOZIE': {
            'host_groups': {'master2': ['OOZIE_SERVER'], 'client': ['OOZIE_CLIENT']},
            'configurations':{'oozie-site': {'oozie.service.JPAService.jdbc.password':'bigdata'}},
            'dependencies': ['HDFS', 'MAPREDUCE2', 'YARN']
        },
        'ACCUMULO': {
            'host_groups': {'master1': ['ACCUMULO_MASTER','ACCUMULO_GC'], 'master2': ['ACCUMULO_MONITOR','ACCUMULO_TRACER'], 'slave': ['ACCUMULO_TSERVER'], 'client': ['ACCUMULO_CLIENT']},
            'dependencies': ['HDFS', 'ZOOKEEPER']
        },
        'DRUID': {
            'host_groups': {'master1': ['DRUID_BROKER','DRUID_COORDINATOR'], 'master2': ['DRUID_ROUTER','DRUID_OVERLORD'], 'slave': ['DRUID_MIDDLEMANAGER','DRUID_HISTORICAL']},
            'dependencies': ['HDFS']
        },
        'SQOOP': {
            'host_groups': {'client': ['SQOOP']},
            'dependencies': ['HDFS', 'MAPREDUCE2']
        },
        'ATLAS': {
            'host_groups': {'master1': ['ATLAS_SERVER'], 'client': ['ATLAS_CLIENT']},
            'configurations':{'atlas-env': {'atlas.admin.password':'bigdata'}},
            'dependencies': ['HBASE', 'HDFS', 'AMBARI_INFRA_SOLR', 'KAFKA']
        },
        'KERBEROS': {
            'host_groups': {'client': ['KERBEROS_CLIENT']}
        },
        'KNOX': {
            'host_groups': {'master2': ['KNOX_GATEWAY']},
            'configurations':{'knox-env': {'knox_master_secret':'bigdata'}}
        },
        'RANGER': {
            'host_groups': {'master1': ['RANGER_ADMIN'], 'master2': ['RANGER_USERSYNC','RANGER_TAGSYNC']},
            'dependencies': ['AMBARI_INFRA_SOLR']
        },
        'RANGER_KMS': {
            'host_groups': {'master2': ['RANGER_KMS_SERVER']},
            'dependencies': ['HDFS']
        }
    }
    service_groups={
        'minimal': ['HDFS','ZOOKEEPER','YARN','MAPREDUCE2'],
        'typical': ['HDFS','ZOOKEEPER','YARN','MAPREDUCE2', 'AMBARI_INFRA_SOLR', 'AMBARI_METRICS', 'HBASE', 'PIG'],
        'all': [key for key in topology]
    }
    @property
    def blueprint(self):
        for b in self.client.blueprints:
            if b.stack==self: return b
        return None
    def register_blueprint(self, data, name=None, validate_topology=True):
        data['Blueprints']={'stack_name': self.name, 'stack_version': self.version}
        if not name: name=self.id
        b=Blueprint(client=self.client,name=name)
        url=b.url
        if not validate_topology: url+='?validate_topology=false'
        self.client.create(url, data=data)
        return self.blueprint
    def _complete_services(self, services):
        srvs=[]
        for s in services:
            srvs.append(s)
            if 'dependencies' in self.topology[s]:
                for d in self.topology[s]['dependencies']:
                    if not d in srvs:
                        srvs.append(d)
        return srvs
    def _join_group(self, services, group_name):
        group=[]
        for s in services:
            if group_name not in self.topology[s]['host_groups']: continue
            group+=[{'name': c} for c in self.topology[s]['host_groups'][group_name]]
        return group
    def build_blueprint(self, services):
        services=self._complete_services(services)
        data={
            'host_groups': [
            {'name': 'master1', 'cardinality': '1', 'components': self._join_group(services, 'master1')},
            {'name': 'master2', 'cardinality': '1', 'components': self._join_group(services, 'master2')},
            {'name': 'slave', 'cardinality': '1+', 'components': self._join_group(services, 'slave')}
            ],
            'configurations': []
        }
        data['host_groups'][-1]['components']+=self._join_group(services, 'client')
        data['host_groups'][0]['components']+=data['host_groups'][-1]['components']
        data['host_groups'][1]['components']+=data['host_groups'][-1]['components']
        for s in services:
            if 'configurations' in self.topology[s]:
                for key in self.topology[s]['configurations']:
                    value=self.topology[s]['configurations'][key]
                    data['configurations'].append({key: {"properties": value}})
        return data
    def register_blueprint_minimal_triple(self):
        data=self.build_blueprint(self.service_groups['minimal'])
        return self.register_blueprint(data)
    def register_blueprint_minimal_single(self):
        minimal_services=self.service_groups['minimal']
        data={
            "host_groups" : [{"name" : "master", "components" : self._join_group(minimal_services, 'master1')+self._join_group(minimal_services, 'master2')+self._join_group(minimal_services, 'slave')+self._join_group(minimal_services, 'client')}]
        }
        return self.register_blueprint(data)
    def register_blueprint_typical_triple(self):
        data=self.build_blueprint(self.service_groups['typical'])
        return self.register_blueprint(data)
    def register_blueprint_triple(self):
        data=self.build_blueprint(self.service_groups['all'])
        return self.register_blueprint(data)

class Blueprint(object):
    def __init__(self,client,name):
        self.client=client
        self.name=name
        self.url='/blueprints/'+self.name
        self._info=None
    @property
    def info(self):
        if not self._info:
            self._info=self.client.get(self.url)
        return self._info
    def delete(self):
        return self.client.delete(self.url)
    @property
    def stack(self):
        ret = self.client.get_stack(name=self.info['Blueprints']['stack_name'], version=self.info['Blueprints']['stack_version'])
        if not ret: raise Exception('stack not found')
        return ret

class Service(object):
    def __init__(self,stack,name):
        self.stack=stack
        self.name=name
        self.url=self.stack.url+"/services/"+self.name
        self._info=None
        self._components=None
        self._categories=None
        self._dependencies=None
        self._quicklinks=None
    @property
    def info(self):
        if not self._info:
            self._info=self.stack.client.get(self.url)
        return self._info
    @property
    def description(self):
        return self.info['StackServices']['comments']
    @property
    def components(self):
        if self._components is None:
            self._components=[]
            for cpn in self.info['components']:
                self._components.append(Component(service=self,name=cpn['StackServiceComponents']['component_name']))
        return self._components
    def get_component(self,name):
        for c in self.components:
            if c.name==name:
                return c
        return None
    @property
    def quicklinks(self):
        if self._quicklinks is None:
            self._quicklinks=[]
            quick_info=self.stack.client._request(self.info['quicklinks'][0]['href'])['QuickLinkInfo']['quicklink_data']['QuickLinksConfiguration']['configuration']['links']
            for link in quick_info:
                self._quicklinks.append(Quicklink(
                    component=self.get_component(link['component_name']),
                    url="http"+link['url'].split("%@")[1]+"%@"+link['url'].split("%@")[2]+link['port']['http_default_port']+link['url'].split("%@")[3],
                    name=link['name'],
                    http_default_port=link['port']['http_default_port']
                ))
        return self._quicklinks
    @property
    def categories(self):
        if self._categories is None:
            self._categories={}
            for c in self.components:
                if not c.category in self._categories:
                    self._categories[c.category]=[]
                self._categories[c.category].append(c)
        return self._categories
    @property
    def masters(self):
        if 'MASTER' in self.categories:
            return self.categories['MASTER']
        return []
    @property
    def slaves(self):
        if 'SLAVE' in self.categories:
            return self.categories['SLAVE']
        return []
    @property
    def clients(self):
        if 'CLIENT' in self.categories:
            return self.categories['CLIENT']
        return []
    @property
    def dependencies(self):
        if self._dependencies is None:
            self._dependencies=[]
            for c in self.components:
                for d in c.dependencies:
                    if d.service not in self._dependencies:
                        self._dependencies.append(d.service)
        return self._dependencies

class Component(object):
    def __init__(self,service,name):
        self.service=service
        self.name=name
        self.url=self.service.url+'/components/'+self.name
        self._info=None
        self._dependencies=None
    @property
    def info(self):
        if not self._info:
            self._info=self.service.stack.client.get(self.url)
        return self._info
    @property
    def display_name(self):
        return self.info['StackServiceComponents']['display_name']
    @property
    def category(self):
        return self.info['StackServiceComponents']['component_category']
    @property
    def dependencies(self):
        if self._dependencies is None:
            self._dependencies=[]
            for d in self.info['dependencies']:
                self._dependencies.append(self.service.stack.get_component(d['Dependencies']['component_name']))
        return self._dependencies
        
class Quicklink(object):
    def __init__(self,component,name,url,http_default_port):
        self.component=component
        self.name=name
        self.url=url
        self.http_default_port=http_default_port
        
    





class Cluster(object):
    def __init__(self,client,name):
        self._info=None
        self._stack=None
        self._hosts=None
        self._services=None
        self.client=client
        self.name=name
        self.url='/clusters/'+name
    def delete(self):
        for h in self.hosts:
            h.delete()
        return self.client.delete(self.url)
    @property
    def requests(self):
        rs=[]
        for r in self.client.get(self.url+'/requests')['items']:
            rs.append(Request(id=r['Requests']['id'],cluster=self))
        return rs
    @property
    def info(self):
        if not self._info:
            self._info=self.client.get(self.url)
        return self._info
    @property
    def stack(self):
        if not self._stack:
            stack_info=self.info['stack_versions'][0]['ClusterStackVersions']
            for s in self.client.stacks:
                if s.name==stack_info['stack'] and s.version==stack_info['version']:
                    self._stack=s
                    break
        return self._stack
    @property
    def blueprint_info(self):
        return self.client.get(self.url+'?format=blueprint')
    @property
    def services(self):
        if self._services is None:
            self._services=[]
            for s in self.info['services']:
                self._services.append(Service(cluster=self,name=s['ServiceInfo']['service_name']))
        return self._services
    @property
    def hosts(self):
        if self._hosts is None:
            self._hosts=[]
            for h in self.info['hosts']:
                self._hosts.append(Host(cluster=self,name=h['Hosts']['host_name']))
        return self._hosts
    def get_host(self,name):
        for h in self.hosts:
            if h.name==name:
                return h
        return None
    def get_service(self,name):
        for s in self.services:
            if s.name==name:
                return s
        return None
    def start(self):
        data={"RequestInfo":{"context":"_PARSE_.START.ALL_SERVICES","operation_level":{"level":"CLUSTER","cluster_name":"'+self.name+'"}},"Body":{"ServiceInfo":{"state":"STARTED"}}}
        return self.client.put(self.url+"/services",data,status_code=202,bad_code_retry=True)
    def stop(self):
        data={"RequestInfo":{"context":"_PARSE_.STOP.ALL_SERVICES","operation_level":{"level":"CLUSTER","cluster_name":"'+self.name+'"}},"Body":{"ServiceInfo":{"state":"INSTALLED"}}}
        return self.client.put(self.url+"/services",data,status_code=202)
    def add_hosts(self,host_groups):
        data=[]
        for hname in host_groups:
            data.append({
                "blueprint" : 'default',
                "host_group" : host_groups[hname],
                "host_name" : hname
            })
        self.client.create(self.url+'/hosts',data=data)

class Request(object):
    def __init__(self,cluster,id):
        self.cluster=cluster
        self.id=id
        self.url=self.cluster.url+"/requests/{}".format(self.id)
        self._tasks=None
        self._info=None
        self._description=None
    def __str__(self):
        return 'req {} || {} || {}'.format(self.id, self.description, self.status)
    @property
    def info(self):
        inf=self.cluster.client.get(self.url) if not self._info else self._info
        if not self._info and inf['Requests']['request_status'] not in ['IN_PROGRESS', 'PENDING']:
            self._info=inf
        return inf
    @property
    def status(self):
        return self.info['Requests']['request_status']
    @property
    def description(self):
        return self.info['Requests']['request_context']
    @property
    def tasks(self):
        if not self._tasks:
            self._tasks=[]
        for t in self.info['tasks']:
            self._tasks.append(Task(self, t['Tasks']['id']))
        return self._tasks

class Task(object):
    def __init__(self,request,id):
        self.request=request
        self.id=id
        self.url=self.request.url+"/tasks/{}".format(self.id)
        self._info=None
        self._description=None
    def __str__(self):
        return 'req: {} || tsk: {} || {} || {}'.format(self.request.id, self.id, self.description, self.status)
    @property
    def info(self):
        inf=self.request.cluster.client.get(self.url) if not self._info else self._info
        if not self._info and inf['Tasks']['status'] not in ['IN_PROGRESS', 'PENDING']:
            self._info=inf
        return inf
    @property
    def status(self):
        return self.info['Tasks']['status']
    @property
    def description(self):
        return self.info['Tasks']['command_detail']

class Service(object):
    def __init__(self,cluster,name):
        self._info=None
        self.cluster=cluster
        self.name=name
        self.url=self.cluster.url+"/services/"+self.name
    @property
    def info(self):
        if not self._info:
            self._info=self.cluster.client.get(self.url)
        return self._info
    @property
    def components(self):
        cs=[]
        for c in self.info['components']:
            cs.append(ServiceComponent(service=self,name=c['ServiceComponentInfo']['component_name']))
        return cs
    def get_component(self,name):
        for c in self.components:
            if c.name==name:
                return c
        return None                     
    @property
    def quicklinks(self):
        qs=[]      
        for q in self.cluster.stack.get_service(self.name).quicklinks:  
            for c in self.components:
                if q.component.name == c.name: 
                    qs.append(QuickUrl(
                        component=self.get_component(q.component.name),
                        url=q.url.replace("%@",c.host_components[-1].host.name)
                    ))
        return qs 
    @property
    def status(self):
        return self.cluster.client.get(self.url+'?fields=ServiceInfo/state')['ServiceInfo']['state']
    def start(self):
        if self.maintenance_status=='ON':
            self.maintenance_off()
        data={"RequestInfo": {"context" :"Start service"}, "Body": {"ServiceInfo": {"state": "STARTED"}}}
        return self.cluster.client.put(self.url,data,status_code=202)
    def stop(self):
        data={"RequestInfo": {"context" :"Stop service"}, "Body": {"ServiceInfo": {"state": "INSTALLED"}}}
        return self.cluster.client.put(self.url,data,status_code=202)
    @property
    def maintenance_status(self):
        return self.info['ServiceInfo']['maintenance_state']
    def maintenance_on(self):
        data={"RequestInfo":{"context":"Turn on Maintenance for '+self.name+'"},"Body":{"ServiceInfo":{"maintenance_state":"ON"}}}
        return self.cluster.client.put(self.url,data)
    def maintenance_off(self):
        data={"RequestInfo":{"context":"Turn off Maintenance for '+self.name+'"},"Body":{"ServiceInfo":{"maintenance_state":"OFF"}}}
        return self.cluster.client.put(self.url,data)
    
class ServiceComponent(object):
    def __init__(self,service,name):
        self.service=service
        self.name=name
        self.url=self.service.url+'/components/'+self.name
    @property
    def info(self):
        return self.service.cluster.client.get(self.url)
    @property
    def host_components(self):
        hs=[]
        for c in self.info['host_components']:
            hs.append(HostComponent(name=c['HostRoles']['component_name'],host=self.service.cluster.get_host(c['HostRoles']['host_name'])))
        return hs

class Host(object):
    def __init__(self,cluster,name,group=None):
        self.cluster=cluster
        self.name=name
        self.group=group
        self.url=self.cluster.url+"/hosts/"+self.name
    @property
    def info(self):
        return self.cluster.client.get(self.url)
    @property
    def metrics(self):
        return self.cluster.client.get(self.url+'?fields=metrics')
    @property
    def components(self):
        if 'host_components' not in self.info: return ()
        cmpns=[]
        for c in self.info['host_components']:
            cmpns.append(HostComponent(host=self,name=c['HostRoles']['component_name']))
        return cmpns
    def register(self):
        self.cluster.client.create(self.url)
        self.cluster._hosts.append(self)
    def delete(self):
        for c in self.components:
            c.delete()
        self.cluster.client.delete(self.url)
        self.cluster._hosts.remove(self)
    def clone(self,from_host):
        print('host_clone({},{})'.format(from_host.name,self.host.name))
        self.register()
        for c in from_host.components:
            cpn=HostComponent(host=self, name=c.name)
            Thread(target=cpn.install).start()

class HostComponent(object):
    def __init__(self,host,name):
        self._info=None
        self.host=host
        self.name=name
        self.url=self.host.url+'/host_components/'+self.name
    @property
    def info(self):
        if not self._info:
            self._info=self.host.cluster.client.get(self.url)
        return self._info
    @property
    def status(self):
        url=self.host.cluster.url+'/components?ServiceComponentInfo/component_name='+self.name+'&fields=host_components/HostRoles/state'
        ret = self.host.cluster.client.get(url)['items'][0]['host_components']
        for cpn_info in ret:
            cpn_info=cpn_info['HostRoles']
            if cpn_info['host_name']==self.host.name:
                return cpn_info['state']
        return None
    @property
    def service_name(self):
        return self.info['HostRoles']['service_name']
    @property
    def install(self):
        self.host.cluster.client.create(self.url)
        data={"RequestInfo":{"context":"Install '+self.name+'","operation_level":{"level":"HOST_COMPONENT","cluster_name":"'+self.host.cluster.name+'","host_name":"'+self.host.name+'","service_name":"'+self.service_name+'"}},"Body":{"HostRoles":{"state":"INSTALLED"}}}
        return self.host.cluster.client.put(self.url,data=data,status_code=202)
    def start(self):
        data={"RequestInfo": {"context" :"Start Component"}, "Body": {"HostRoles": {"state": "STARTED"}}}
        return self.host.cluster.client.put(self.url,data=data,status_code=202)
    def stop(self):
        data={"RequestInfo":{"context":"Stop Component"},"Body":{"HostRoles":{"state":"INSTALLED"}}}
        return self.host.cluster.client.put(self.url,data=data,status_code=202)
    def delete(self):
        if self.status=='STARTED':
            self.stop()
        return self.host.cluster.client.delete(self.url)
class QuickUrl(object):  
    def __init__(self,component,url):
        self.component=component
        self.url=url
        
        


import argparse
from .client import Client

parser = argparse.ArgumentParser(description='ambari cmd line.')
parser.add_argument('server', metavar='S', nargs=1,
                    help='the ambari server host:port to connect.')
parser.add_argument('target', choices=['stack', 'cluster', 'host', 'service', 'request'], metavar='T', nargs=1,
                    help='the target resource to perform an action on.')
parser.add_argument('action', metavar='A', nargs=1,
                    help='the action to be performed on the target resource.')
parser.add_argument('opts', metavar='O', nargs='*',
                    help='the extra argument for the action.')
args = parser.parse_args()

client=Client('http://'+args.server[0])

def run():
    func=globals()['{}_{}'.format(args.target[0],args.action[0])]
    func()

def cluster_create():
    (name,size)=args.opts[0:2]
    hosts=args.opts[2:]
    client.create_cluster(name,hosts=hosts,size=size)

def cluster_create_from_vdf():
    (VDF_url,name,size)=args.opts[0:3]
    hosts=args.opts[3:]
    client.create_cluster(name,hosts=hosts,VDF_url=VDF_url,size=size)

def host_clone():
    from_h=client.cluster.get_host(args.opts[0])
    to_h=client.cluster.get_host(args.opts[1])
    to_h.clone(from_h)

def host_delete():
    client.cluster.get_host(args.opts[0]).delete()

def service_start():
    if args.opts:
        client.cluster.get_service(name=args.opts[0]).start()
    else:
        client.cluster.start()

def service_stop():
    if args.opts:
        client.cluster.get_service(name=args.opts[0]).stop()
    else:
        client.cluster.stop()

def service_list():
    for s in client.cluster.services:
        print(s.name)

def request_list():
    for r in reversed(client.cluster.requests):
        print(r)
        for t in r.tasks:
            print(t)
        if r.status not in ['IN_PROGRESS', 'PENDING']: break
